@@ -3,6 +3,7 @@ use crate::expression::{
     deep::{DeepBuf, DeepEx, ExprIdxVec},
     partial_derivatives, Express,
 };
+use crate::data_type::DataType;
 use crate::{DefaultOpsFactory, ExError, ExResult, MakeOperators};
 use num::Float;
 use std::fmt::{self, Debug, Display, Formatter};
@@ -38,7 +39,7 @@ use std::str::FromStr;
 #[derive(Clone, Eq, PartialEq, Ord, PartialOrd, Debug)]
 pub struct FlatEx<'a, T, OF = DefaultOpsFactory<T>>
 where
-    T: Copy + Debug,
+    T: Clone + Debug,
     OF: MakeOperators<T>,
 {
     nodes: FlatNodeVec<T>,
@@ -51,7 +52,7 @@ where
 
 impl<'a, T, OF> FlatEx<'a, T, OF>
 where
-    T: Copy + Debug,
+    T: Clone + Debug,
     OF: MakeOperators<T>,
 {
     fn flatten(deepex: DeepEx<'a, T>) -> Self {
@@ -71,13 +72,13 @@ where
 
 impl<'a, T, OF> Express<'a, T> for FlatEx<'a, T, OF>
 where
-    T: Copy + Debug,
+    T: DataType,
     OF: MakeOperators<T>,
 {
     fn from_str(text: &'a str) -> ExResult<Self>
     where
         <T as std::str::FromStr>::Err: Debug,
-        T: Copy + FromStr,
+        T: DataType,
     {
         let ops = OF::make();
         let deepex = DeepEx::from_ops(text, &ops)?;
@@ -87,7 +88,7 @@ where
     fn from_pattern(text: &'a str, number_regex_pattern: &str) -> ExResult<Self>
     where
         <T as std::str::FromStr>::Err: Debug,
-        T: Copy + FromStr + Debug,
+        T: DataType,
     {
         let ops = OF::make();
         let deepex = DeepEx::from_pattern(text, &ops, number_regex_pattern)?;
@@ -140,7 +141,7 @@ where
 /// The expression is displayed as a string created by [`unparse`](FlatEx::unparse).
 impl<'a, T, OF> Display for FlatEx<'a, T, OF>
 where
-    T: Copy + Debug,
+    T: DataType,
     OF: MakeOperators<T>,
 {
     fn fmt(&self, f: &mut Formatter) -> fmt::Result {
@@ -174,7 +175,7 @@ where
 #[derive(Clone, Eq, PartialEq, Ord, PartialOrd, Debug)]
 pub struct OwnedFlatEx<T, OF = DefaultOpsFactory<T>>
 where
-    T: Copy + Debug,
+    T: Clone + Debug,
     OF: MakeOperators<T>,
 {
     deepex_buf: Option<DeepBuf<T>>,
@@ -186,7 +187,7 @@ where
 }
 impl<T, OF> OwnedFlatEx<T, OF>
 where
-    T: Copy + Debug,
+    T: Clone + Debug,
     OF: MakeOperators<T>,
 {
     /// Creates an `OwnedFlatEx` instance from an instance of `FlatEx`.
@@ -203,13 +204,13 @@ where
 }
 impl<'a, T, OF> Express<'a, T> for OwnedFlatEx<T, OF>
 where
-    T: Copy + Debug,
+    T: DataType,
     OF: MakeOperators<T>,
 {
     fn from_str(text: &'a str) -> ExResult<Self>
     where
         <T as std::str::FromStr>::Err: Debug,
-        T: Copy + FromStr,
+        T: Clone + FromStr,
     {
         Ok(Self::from_flatex(FlatEx::from_str(text)?))
     }
@@ -217,7 +218,7 @@ where
     fn from_pattern(text: &'a str, number_regex_pattern: &str) -> ExResult<Self>
     where
         <T as std::str::FromStr>::Err: Debug,
-        T: Copy + FromStr + Debug,
+        T: DataType,
     {
         Ok(Self::from_flatex(FlatEx::from_pattern(
             text,
@@ -274,7 +275,7 @@ where
 /// The expression is displayed as a string created by [`unparse`](OwnedFlatEx::unparse).
 impl<T, OF> Display for OwnedFlatEx<T, OF>
 where
-    T: Copy + Debug,
+    T: DataType,
     OF: MakeOperators<T>,
 {
     fn fmt(&self, f: &mut Formatter) -> fmt::Result {
