@@ -1,12 +1,12 @@
-use std::{fmt, fmt::Debug, marker::PhantomData, str::FromStr};
+use std::{fmt, fmt::Debug, marker::PhantomData};
 
-use num::Float;
 use serde::{de, de::Visitor, Deserialize, Deserializer, Serialize, Serializer};
 
+use crate::data_type::DataType;
 use crate::OwnedFlatEx;
 use crate::{prelude::*, MakeOperators};
 
-fn serialize<'a, T: Copy, S: Serializer, Ex: Express<'a, T>>(
+fn serialize<'a, T: Clone, S: Serializer, Ex: Express<'a, T>>(
     serializer: S,
     expr: &Ex,
 ) -> Result<S::Ok, S::Error> {
@@ -17,7 +17,7 @@ fn serialize<'a, T: Copy, S: Serializer, Ex: Express<'a, T>>(
     )
 }
 
-impl<'de: 'a, 'a, T: Copy + Debug, OF: MakeOperators<T>> Serialize for FlatEx<'a, T, OF> {
+impl<'de: 'a, 'a, T: DataType, OF: MakeOperators<T>> Serialize for FlatEx<'a, T, OF> {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -26,7 +26,7 @@ impl<'de: 'a, 'a, T: Copy + Debug, OF: MakeOperators<T>> Serialize for FlatEx<'a
     }
 }
 
-impl<'de: 'a, 'a, T: Copy + Debug + FromStr + 'a, OF: MakeOperators<T>> Deserialize<'de>
+impl<'de: 'a, 'a, T: DataType + 'a, OF: MakeOperators<T>> Deserialize<'de>
     for FlatEx<'a, T, OF>
 where
     <T as std::str::FromStr>::Err: Debug,
@@ -48,8 +48,7 @@ struct FlatExVisitor<'a, T, OF> {
     of_dummy: PhantomData<OF>,
 }
 
-impl<'de: 'a, 'a, T: Copy + Debug + FromStr, OF: MakeOperators<T>> Visitor<'de>
-    for FlatExVisitor<'a, T, OF>
+impl<'de: 'a, 'a, T: DataType, OF: MakeOperators<T>> Visitor<'de> for FlatExVisitor<'a, T, OF>
 where
     <T as std::str::FromStr>::Err: Debug,
 {
@@ -68,7 +67,7 @@ where
     }
 }
 
-impl<'de, T: Copy + Debug, OF: MakeOperators<T>> Serialize for OwnedFlatEx<T, OF> {
+impl<'de, T: DataType, OF: MakeOperators<T>> Serialize for OwnedFlatEx<T, OF> {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -77,7 +76,7 @@ impl<'de, T: Copy + Debug, OF: MakeOperators<T>> Serialize for OwnedFlatEx<T, OF
     }
 }
 
-impl<'de, T: Float + Debug + FromStr, OF: MakeOperators<T>> Deserialize<'de> for OwnedFlatEx<T, OF>
+impl<'de, T: DataType, OF: MakeOperators<T>> Deserialize<'de> for OwnedFlatEx<T, OF>
 where
     <T as std::str::FromStr>::Err: Debug,
 {
@@ -98,8 +97,7 @@ struct OwnedFlatExVisitor<T, OF> {
     another_dummy: PhantomData<OF>,
 }
 
-impl<'de, T: Float + Debug + FromStr, OF: MakeOperators<T>> Visitor<'de>
-    for OwnedFlatExVisitor<T, OF>
+impl<'de, T: DataType, OF: MakeOperators<T>> Visitor<'de> for OwnedFlatExVisitor<T, OF>
 where
     <T as std::str::FromStr>::Err: Debug,
 {
