@@ -1,4 +1,4 @@
-#![doc(html_root_url = "https://docs.rs/exmex/0.11.3")]
+#![doc(html_root_url = "https://docs.rs/exmex/0.11.4")]
 //! Exmex is a fast, simple, and **ex**tendable **m**athematical **ex**pression evaluator
 //! with the ability to compute [partial derivatives](FlatEx::partial) of expressions.  
 //!
@@ -218,8 +218,8 @@
 //! ```
 //! ### Custom Data Types of Numbers
 //!
-//! You can use any type that implements [`Copy`](core::marker::Copy) and
-//! [`FromStr`](std::str::FromStr). In case the representation of your data type in the
+//! You can use any type that implements [`Clone`](core::marker::Clone),
+//! [`FromStr`](std::str::FromStr), and [`Debug`](std::fmt::Debug). In case the representation of your data type in the
 //! string does not match the number regex `r"\.?[0-9]+(\.[0-9]+)?"`, you have to pass a
 //! suitable regex and use the function
 //! [`from_pattern`](Express::from_pattern) instead of [`parse`](crate::parse) or
@@ -307,7 +307,7 @@ use std::{fmt::Debug, str::FromStr};
 
 use expression::deep::DeepEx;
 use num::Float;
-
+use data_type::DataType;
 mod definitions;
 mod expression;
 #[macro_use]
@@ -315,6 +315,7 @@ mod operators;
 mod parser;
 mod result;
 mod util;
+mod data_type;
 pub use {
     expression::{
         flat::{FlatEx, OwnedFlatEx},
@@ -337,7 +338,7 @@ pub mod prelude {
 /// In case the parsing went wrong, e.g., due to an invalid input string, an
 /// [`ExError`](ExError) is returned.
 ///
-pub fn eval_str<T: Float + FromStr + Debug>(text: &str) -> ExResult<T>
+pub fn eval_str<T: Float + DataType>(text: &str) -> ExResult<T>
 where
     <T as FromStr>::Err: Debug,
 {
@@ -358,7 +359,7 @@ where
 /// In case the parsing went wrong, e.g., due to an invalid input string, an
 /// [`ExError`](ExError) is returned.
 ///
-pub fn parse<T: Float + FromStr + Debug>(text: &str) -> ExResult<FlatEx<T>>
+pub fn parse<T: Float + DataType>(text: &str) -> ExResult<FlatEx<T>>
 where
     <T as FromStr>::Err: Debug,
 {
@@ -484,7 +485,7 @@ mod tests {
         let expr = FlatEx::<f64>::from_str(sut).unwrap();
         assert_float_eq_f64(expr.eval(&[1.5707963267948966]).unwrap(), 1.0);
 
-        #[derive(Clone, Copy, Debug, PartialEq, Eq)]
+        #[derive(Clone, Debug, PartialEq, Eq)]
         struct Thumbs {
             val: bool,
         }
